@@ -168,7 +168,8 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         return volume_paths
 
     @utils.trace
-    @synchronized('extend_volume')
+    @synchronized('extend_volume', external=True)
+    @utils.connect_volume_undo_prepare_result
     def extend_volume(self, connection_properties):
         """Update the local kernel's size information.
 
@@ -189,7 +190,8 @@ class FibreChannelConnector(base.BaseLinuxConnector):
             raise exception.VolumePathsNotFound()
 
     @utils.trace
-    @synchronized('connect_volume')
+    @utils.connect_volume_prepare_result
+    @synchronized('connect_volume', external=True)
     def connect_volume(self, connection_properties):
         """Attach the volume to instance_name.
 
@@ -314,7 +316,8 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         return raw_devices
 
     @utils.trace
-    @synchronized('connect_volume')
+    @synchronized('connect_volume', external=True)
+    @utils.connect_volume_undo_prepare_result(unlink_after=True)
     def disconnect_volume(self, connection_properties, device_info,
                           force=False, ignore_errors=False):
         """Detach the volume from instance_name.
@@ -329,7 +332,7 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         target_wwn - World Wide Name
         target_lun - LUN id of the volume
         """
-
+        exc = exception.ExceptionChainer()
         devices = []
         wwn = None
 
@@ -345,19 +348,34 @@ class FibreChannelConnector(base.BaseLinuxConnector):
                 wwn = self._linuxscsi.get_scsi_wwn(path)
                 mpath_path = self._linuxscsi.find_multipath_device_path(wwn)
                 if mpath_path:
-                    self._linuxscsi.flush_multipath_device(mpath_path)
+                    with exc.context(force, 'Flushing %s failed', mpath_path):
+                        self._linuxscsi.flush_multipath_device(mpath_path)
             dev_info = self._linuxscsi.get_device_info(real_path)
             devices.append(dev_info)
 
-        LOG.debug("devices to remove = %s", devices)
-        self._remove_devices(connection_properties, devices, device_info)
+        # If flush failed, then remove it forcefully since force=True
+        if mpath_path and exc:
+            with exc.context(force, 'Removing multipath %s failed',
+                             mpath_path):
+                mpath_name = os.path.basename(os.path.realpath(mpath_path))
+                self._linuxscsi.multipath_del_map(mpath_name)
 
-    def _remove_devices(self, connection_properties, devices, device_info):
+        LOG.debug("devices to remove = %s", devices)
+        self._remove_devices(connection_properties, devices, device_info,
+                             force, exc)
+
+        if exc:  # type: ignore
+            LOG.warning('There were errors removing %s, leftovers may remain '
+                        'in the system', volume_paths)
+            if not ignore_errors:
+                raise exc  # type: ignore
+
+    def _remove_devices(self, connection_properties, devices, device_info,
+                        force, exc):
         # There may have been more than 1 device mounted
         # by the kernel for this volume.  We have to remove
         # all of them
-        path_used = self._linuxscsi.get_dev_path(connection_properties,
-                                                 device_info)
+        path_used = utils.get_dev_path(connection_properties, device_info)
         # NOTE: Due to bug #1897787 device_info may have a real path for some
         # single paths instead of a symlink as it should have, so it'll only
         # be a multipath if it was a symlink (not real path) and it wasn't a
@@ -368,11 +386,12 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         # paths, whereas for multipaths we have multiple link formats.
         was_multipath = '/pci-' not in path_used and was_symlink
         for device in devices:
-            device_path = device['device']
-            flush = self._linuxscsi.requires_flush(device_path,
-                                                   path_used,
-                                                   was_multipath)
-            self._linuxscsi.remove_scsi_device(device_path, flush=flush)
+            with exc.context(force, 'Removing device %s failed', device):
+                device_path = device['device']
+                flush = self._linuxscsi.requires_flush(device_path,
+                                                       path_used,
+                                                       was_multipath)
+                self._linuxscsi.remove_scsi_device(device_path, flush=flush)
 
     def _get_pci_num(self, hba):
         # NOTE(walter-boring)
